// Copyright 2018 the Charts project authors. Please see the AUTHORS file
// for details.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:charts_common/common.dart' as common
    show
        AxisSpec,
        BarChart,
        BarGroupingType,
        BarRendererConfig,
        BarRendererDecorator,
        RTLSpec,
        Series,
        SeriesRendererConfig;
import 'behaviors/domain_highlighter.dart' show DomainHighlighter;
import 'behaviors/chart_behavior.dart' show ChartBehavior;
import 'package:meta/meta.dart' show immutable;
import 'base_chart.dart' show LayoutConfig;
import 'base_chart_state.dart' show BaseChartState;
import 'cartesian_chart.dart' show CartesianChart;
import 'selection_model_config.dart' show SelectionModelConfig;
import 'symbol_renderer.dart' show RoundedRectSymbolRenderer;

@immutable
class BarChart<T> extends CartesianChart<T, String> {
  final bool vertical;
  final common.BarRendererDecorator barRendererDecorator;

  BarChart(
    List<common.Series<T, String>> seriesList, {
    bool animate,
    Duration animationDuration,
    common.AxisSpec domainAxis,
    common.AxisSpec primaryMeasureAxis,
    common.AxisSpec secondaryMeasureAxis,
    common.BarGroupingType barGroupingType,
    common.BarRendererConfig<T> defaultRenderer,
    List<common.SeriesRendererConfig<T, String>> customSeriesRenderers,
    List<ChartBehavior> behaviors,
    List<SelectionModelConfig> selectionModels,
    common.RTLSpec rtlSpec,
    this.vertical: true,
    bool defaultInteractions: true,
    LayoutConfig layoutConfig,
    this.barRendererDecorator,
  }) : super(
          seriesList,
          animate: animate,
          animationDuration: animationDuration,
          domainAxis: domainAxis,
          primaryMeasureAxis: primaryMeasureAxis,
          secondaryMeasureAxis: secondaryMeasureAxis,
          defaultRenderer: defaultRenderer ??
              new common.BarRendererConfig<T>(
                  groupingType: barGroupingType,
                  barRendererDecorator: barRendererDecorator,
                  symbolRenderer: new RoundedRectSymbolRenderer()),
          customSeriesRenderers: customSeriesRenderers,
          behaviors: behaviors,
          selectionModels: selectionModels,
          rtlSpec: rtlSpec,
          defaultInteractions: defaultInteractions,
          layoutConfig: layoutConfig,
        );

  @override
  common.BarChart<T> createCommonChart(BaseChartState chartState) =>
      new common.BarChart<T>(
          vertical: vertical, layoutConfig: layoutConfig?.commonLayoutConfig);

  @override
  void addDefaultInteractions(List<ChartBehavior> behaviors) {
    super.addDefaultInteractions(behaviors);

    behaviors.add(new DomainHighlighter());
  }
}
